
clear
%% 校准

BETA = 0.99;
SIGMA = 1.2;
V = 0.5;
MU = 0.6; 
%Emission per unit of Energy
TAU_l = 0.05;
TAU_k = 0.05;
TAU_m = 0.05;
DELTA = 0.025;
ALPHA_sk = 0.7;
ALPHA_sm = 0.1;
ALPHA_pk = 0.3;
ALPHA_py = 0.4;
GAMMA_m = 2.136; 
LAMBDA = -5/0.6;
DELTA_z = 0.005;

ETA0 = 0.0015;
ETA1 = -0.0000065;
ETA2 = 0.00000015;

THETA_s = 2;%Guo,1
THETA_p = 3;%Guo,1
b_s = 0.5;
b_p = 0.5;
RHO_q = 0.5;
RHO_A = 0.95;

RHO_G = 0.95;
RHO_z = 0.95;
A_ss = 1;
q_ss = 1;
G_ss = 1;
Pz_ss= 1;
% l0 = 0.2;
% Rpen_ss = 0.5;


save PARA.mat BETA SIGMA V MU TAU_l TAU_k TAU_m ...
    DELTA ALPHA_sk ALPHA_sm ALPHA_pk ALPHA_py GAMMA_m ...
    LAMBDA DELTA_z ETA0 ETA1 ETA2 THETA_s THETA_p ...
    b_s b_p RHO_q RHO_A  RHO_G RHO_z A_ss q_ss G_ss Pz_ss;


%% 稳态

load PARA.mat

%与K,Pm,M无关的稳态
R_ss = 1/BETA;
Rk_ss  = (1-TAU_k)^(-1)*(1/BETA-(1-DELTA));
Pz_ss = Pz_ss;
er_ss = 1-exp(Pz_ss/LAMBDA);
ts    = (ALPHA_pk/(ALPHA_sk*ALPHA_py)+1)^(-1); %国有资本权重

%与K,Pm,M有关的稳态
syms K Pm M positive

%SOE
Ls_f = ((1-ALPHA_sk-ALPHA_sm)/ALPHA_sk*Rk_ss/(1+b_s)/THETA_s*ts*K)^(1/(1+b_s));
Ws_f = THETA_s*Ls_f^b_s;
M_f  = ALPHA_sm/ALPHA_sk*Rk_ss/(Pm+(1-er_ss)*MU*Pz_ss)*ts*K;
EtaM_f = q_ss*M_f^(GAMMA_m-1);
Z_f  = (1-er_ss)*MU*M_f;
ST_f = Z_f/DELTA_z;
EtaL_f = 1-ETA0-ETA1*ST_f-ETA2*ST_f^2;
ys_f = A_ss*(ts*K)^(ALPHA_sk)*(EtaL_f*Ls_f)^(1-ALPHA_sk-ALPHA_sm)*(EtaM_f*M)^(ALPHA_sm);

%POE
Lp_f = ((1-ALPHA_pk-ALPHA_py)/ALPHA_pk*Rk_ss/(1+b_p)/THETA_p*(1-ts)*K)^(1/(1+b_p));
Wp_f = THETA_p*Lp_f^b_p;
yp_f = A_ss*((1-ts)*K)^ALPHA_pk*(EtaL_f*Lp_f)^(1-ALPHA_pk-ALPHA_py)*ys_f^ALPHA_py;

%3个未知数3个方程
M_RHS = ALPHA_sm/ALPHA_sk*Rk_ss/(Pm+(1-er_ss)*MU*Pz_ss)*ts*K;
a = M - M_RHS;
b = ((1-er_ss)*MU)^(1+V)*M^V*(yp_f-DELTA*K-G_ss)^SIGMA - (1-TAU_m)*Pm;
c = yp_f - Rk_ss/ALPHA_pk*(1-ts)*K;
F = matlabFunction([a;b;c]);
%F solve
eqn1 = a==0;
eqn2 = b==0;
eqn3 = c==0;
eqn = [eqn1 eqn2 eqn3];
%VPA solve

XX = ones(3);
x0 = 100*XX(1,:);
%初值

%F
X_fsolve = fsolve(@(x)F(x(1),x(2),x(3)),x0)
%K_ss = X_fsolve(1);
%Pm_ss = X_fsolve(3);
%M_ss = X_fsolve(2);

%VPA
X_vpa = vpasolve(eqn,x0)
K_ss = double(X_vpa.K);
Pm_ss= double(X_vpa.Pm);
M_ss = double(X_vpa.M);

%检查方程解
if abs(F(K_ss,M_ss,Pm_ss))<1e-5
    disp('方程解检查无误')
end




%% 给出稳态值

%K
Ks_ss = ts*K_ss;
Kp_ss = (1-ts)*K_ss;

%SOE
Ls_ss = ((1-ALPHA_sk-ALPHA_sm)/ALPHA_sk*Rk_ss/(1+b_s)/THETA_s*ts*K_ss)^(1/(1+b_s));
Ws_ss = THETA_s*Ls_ss^b_s;
M_ss  = ALPHA_sm/ALPHA_sk*Rk_ss/(Pm_ss+(1-er_ss)*MU*Pz_ss)*ts*K_ss;
EtaM_ss = q_ss*M_ss^(GAMMA_m-1);
Z_ss  = (1-er_ss)*MU*M_ss;
ST_ss = Z_ss/DELTA_z;
EtaL_ss = 1-ETA0-ETA1*ST_ss-ETA2*ST_ss^2;
ys_ss = A_ss*(ts*K_ss)^(ALPHA_sk)*(EtaL_ss*Ls_ss)^(1-ALPHA_sk-ALPHA_sm)*(EtaM_ss*M_ss)^(ALPHA_sm);
ysd_ss = ys_ss;

%POE
Lp_ss = ((1-ALPHA_pk-ALPHA_py)/ALPHA_pk*Rk_ss/(1+b_p)/THETA_p*(1-ts)*K_ss)^(1/(1+b_p));
Wp_ss = THETA_p*Lp_ss^b_p;
yp_ss = A_ss*((1-ts)*K_ss)^ALPHA_pk*(EtaL_ss*Lp_ss)^(1-ALPHA_pk-ALPHA_py)*ys_ss^ALPHA_py;

%I,C
I_ss = DELTA*K_ss;
C_ss = yp_ss - I_ss - G_ss;
%P
P_ss = ALPHA_py*Rk_ss/ALPHA_pk/ysd_ss*Kp_ss;%P_ss = ALPHA_py*yp_ss/ys_ss;
%T,B
T_ss = TAU_l*(Ws_ss*Ls_ss+Wp_ss*Lp_ss)+TAU_k*Rk_ss*K_ss+TAU_m*Pm_ss*M_ss;
B_ss = (R_ss-1)^(-1)*(T_ss+Pz_ss*Z_ss-G_ss);

%CE
CE_ss = -LAMBDA*MU*M_ss*(er_ss+(1-er_ss)*log(1-er_ss));


save STEADY Ws_ss Wp_ss Pm_ss P_ss Pz_ss C_ss I_ss B_ss...
    T_ss er_ss Z_ss ST_ss CE_ss K_ss M_ss Ks_ss Kp_ss Ls_ss...
    Lp_ss EtaM_ss EtaL_ss ys_ss ysd_ss yp_ss Rk_ss R_ss ...
    A_ss G_ss q_ss
 
%%
clear
dynare Guo_Xiao


%% 稳态检查
M_.endo_names;
oo_.steady_state;
len = length(oo_.steady_state);
Diff = [];
for i=1:len
    eval(['MySS ='  M_.endo_names{i} '_ss;']);
    realSS = oo_.steady_state(i);
    diff = roundn(MySS-realSS,3);
    if abs(diff)>1e-2
        disp(M_.endo_names{i})
    end
    Diff = [Diff diff];
end

if Diff == 0
    disp('稳态求解检查无误')
end






